# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.conf import settings
from rest_framework import status
from rest_framework.exceptions import APIException
from .code import Code, CodeData

__all__ = ["ErrCode", "APIError", "CodeData"]

CodeDefine = (
    ('SUCCESS', '0000', '返回成功'),
    ('ERROR', '0001', '未知错误'),
    ('ERR_SYS_ERROR', '0002', '服务异常'),

    ('ERR_JWT_EMPTY', '1001', '没有找到JWT信息'),
    ('ERR_JWT_EXPIRED', '1002', '签名过期'),
    ('ERR_JWT_DECODE', '1003', 'JWT解码错误'),
    ('ERR_JWT_INVALID', '1004', 'JWT不合法'),

    ('ERR_COMMON_BAD_PARAM', '9001', '参数错误'),
    ('ERR_COMMON_BAD_FORMAT', '9002', '格式错误'),
    ('ERR_COMMON_PERMISSION', '9003', '权限错误'),
    ('ERR_COMMON_NOT_FOUND', '9004', '未找到'),
)

ErrCode = Code(CodeDefine + getattr(settings, "API_ERROR_CODES", ()))


class APIError(ValueError):
    status_code = status.HTTP_400_BAD_REQUEST

    def __init__(self, errcode=ErrCode.ERROR, status_code=status.HTTP_400_BAD_REQUEST, **kwargs):
        self.errcode = errcode
        self.code = errcode.code
        self.kwargs = kwargs
        self.status_code = status_code
        self.message = self.kwargs.get('message', errcode.message)
        super(APIError, self).__init__(self.message)

    def get_res_dict(self):
        return self.errcode.get_res_dict(**self.kwargs)
